@@ -108,15 +108,10 @@ static inline json_create_status_t
 add_str (json_create_t * jc, const char * s)
 {
     int i;
-    i = 0;
-    while (1) {
+    for (i = 0; s[i]; i++) {
 	unsigned char c;
-	if (c == 0) {
-	    return json_create_ok;
-	}
 	c = (unsigned char) s[i];
 	CALL (add_char (jc, c));
-	i++;
     }
     return json_create_ok;
 }
@@ -316,7 +311,7 @@ static json_create_status_t
 json_create_recursively (json_create_t * jc, SV * input)
 {
     if (! SvOK (input)) {
-	CALL (add_str (jc, "null"));
+	ADD ("null");
 	return json_create_ok;
     }
     if (SvROK (input)) {
@@ -348,20 +343,16 @@ json_create_recursively (json_create_t * jc, SV * input)
     else {
 	SV * r = input;
 	if (SvTYPE (r) == SVt_NULL) {
-	    CALL (add_str (jc, "null"));
-	    return json_create_ok;
+	    ADD ("null");
 	}
 	else if (SvTYPE (r) == SVt_PV) {
 	    CALL (json_create_add_string (jc, r));
-	    return json_create_ok;
 	}
 	else if (SvTYPE (r) == SVt_IV) {
 	    CALL (json_create_add_integer (jc, r));
-	    return json_create_ok;
 	}
 	else if (SvTYPE (r) == SVt_NV) {
 	    CALL (json_create_add_float (jc, r));
-	    return json_create_ok;
 	}
 	else {
 	    if (JCEH) {
@@ -378,9 +369,11 @@ json_create_recursively (json_create_t * jc, SV * input)
 	json_create_status_t status;				\
 	status = x;						\
 	if (status != json_create_ok) {				\
-	    fprintf (stderr,					\
-		     "%s:%d: %s failed with status %d\n",	\
-		     __FILE__, __LINE__, #x, status);		\
+	    if (JCEH) {						\
+		(*JCEH) (__FILE__, __LINE__,			\
+			 "%s failed with status %d\n",		\
+			 #x, status);				\
+	    }							\
 	    /* Free the memory of "output". */			\
 	    if (jc.output) {					\
 		SvREFCNT_dec (jc.output);			\
